%
% Jacobi, GS and SOR 
%
clear all;
close all;

n = 32;
H = hilb(n);
xtrue = ones(n,1);
b = H*xtrue;
tol = 1e-4;
IterMax = 200;
x0 = zeros(n,1);

[x,iter,flag] = Jacobi(H,b,x0,tol,IterMax);
if flag==1
    fprintf('Jacobi: Iter=%d, relres=%.4e\n', iter, norm(b-H*x)/norm(b));
else
    disp('Jacobi: ')
end

[x,iter,flag] = GS(H,b,x0,tol,IterMax);
if flag==1
    fprintf('Gauss-Seidel: Iter=%d, relres=%.4e\n', iter, norm(b-H*x)/norm(b));
else
    disp('Gauss-Seidel: ')
end
%%%%% ========== End of the main program ===========================


% Jacobi iteration
function [x,iter,flag] = Jacobi(A,b,x,tol,IterMax)
bnorm = norm(b);
n = length(x);
xold = x;
for k = 1 : IterMax
    for i = 1 : n
        x(i) = ( b(i) - A(i,1:i-1)*xold(1:i-1) - A(i,i+1:n)*xold(i+1:n) ) / A(i,i);
    end
    relres = norm(b-A*x)/bnorm;
    fprintf('k=%d,relres=%.4e\n',k,relres); % pause(0.01)
    if relres < tol
        iter = k; flag = 1; return
    end
    xold = x;
end
iter = k; flag = 0;
end

% Gauss-Seidel iteration
function [x,iter,flag] = GS(A,b,x,tol,IterMax)
bnorm = norm(b);
n = length(x);
for k = 1 : IterMax
    for i = 1 : n
        x(i) = ( b(i) - A(i,1:i-1)*x(1:i-1) - A(i,i+1:n)*x(i+1:n) ) / A(i,i);
    end
    relres = norm(b-A*x)/bnorm;
    if relres < tol
        iter = k; flag = 1; return
    end
end
iter = k; flag = 0;
end

